<?php
/*
Plugin Name: lazyone's most recent user activity plugin
Plugin URI: http://spielwiese.fontein.de/tag/last-user-activity-plugin/
Description: Logs the most recent activity of each logged in user
Version: 1.0
Author: Felix Fontein
Author URI: http://spielwiese.fontein.de
*/

function lazylast_update()
/* This function is called whenever a Wordpress page, feed, ... is requested.
   It checks whether the current user is logged in; in case he is, the timestamp
   is noted. */
{
  if (is_user_logged_in())
  {
    $dt = new DateTime;
    update_option('lazylast-' . wp_get_current_user()->user_login, $dt->format('Y/m/d H:i:s'));
  }
}

function lazylast_compare($a, $b)
/* Compares two users by first comparing the last access timestamp, and then the
   display name. */
{
  $at = ($a->last_activity != "") ? intval($a->last_activity_d->format("U")) : 0;
  $bt = ($b->last_activity != "") ? intval($b->last_activity_d->format("U")) : 0;
  // compare time stampts first
  if ($at < $bt)
    return 1;
  if ($at > $bt)
    return -1;
  // if they are equal, compare display names (case insensitive)
  return strcasecmp($a->display_name, $b->display_name);
}

function lazylast_options()
/* Shows a list of last user accesses. This can only be seen by admins. */
{
  if (!current_user_can('list_users'))
    return;
  // retrieve user list from database
  global $wpdb;
  $users = $wpdb->get_results("SELECT * FROM " . $wpdb->users);
  // add last login
  foreach ($users as $user)
  {
    $user->last_activity = get_option('lazylast-' . $user->user_login);
    $user->last_activity_d = new DateTime($user->last_activity);
  }
  // sort
  usort($users, lazylast_compare);
  // output
  echo "<h3>Last user activity:</h3>";
  echo "<table>";
  $n = 1;
  echo "<tr><th align='left'>#</th><th width='150px' align='left'>name.</th><th align='left'>most recent activity.</th></tr>";
  $dt = new DateTime();
  $now = $dt->format('U');
  foreach ($users as $user)
  {
    echo "<tr><td valign='top'>" . $n . ".</td>";
    echo "<td>" . htmlentities($user->display_name) . "<br />";
    echo "<small>edit <a href='" . get_admin_url() . "user-edit.php?user_id=" . $user->ID . "'>" . htmlentities($user->user_login) . "</a></small></td>";
    echo "<td valign='top'>";
    if ($user->last_activity != "")
    {
      echo $user->last_activity_d->format("d.m.Y H:i:s P");
      $dist = $now - $user->last_activity_d->format("U");
      echo ", i.e. ";
      if ($dist < 60)
        echo $dist . " seconds";
      else if ($dist < 60 * 10)
        echo round($dist / 60, 1) . " minutes";
      else if ($dist < 60 * 60)
        echo floor($dist / 60) . " minutes";
      else if ($dist < 60 * 60 * 10)
        echo round($dist / 60 / 60, 1) . " hours";
      else if ($dist < 60 * 60 * 24)
        echo floor($dist / 60 / 60) . " hours";
      else if ($dist < 60 * 60 * 24 * 10)
        echo round($dist / 60 / 60 / 24, 1) . " days";
      else if ($dist < 60 * 60 * 24 * 31)
        echo floor($dist / 60 / 60 / 24) . " days";
      else
        echo floor($dist / 6 / 60 / 24 / 305) . " months";
      echo " ago";
    }
    else
    {
      echo "<i style='color: gray;'>never logged in</i>";
    }
    echo "</td></tr>";
    $n++;
  }
  echo "</table>";
}

function lazylast_menu()
/* Adds an option to show the list of last user accesses to the admin panel. */
{
  add_options_page('Last User Activity.', 'Last user activity', 'manage_options', 'lazylast', lazylast_options);
}

function lazylast_init()
/* Initializes the plugin. */
{
  add_action('wp_head', lazylast_update);
  add_action('rss_head', lazylast_update);
  add_action('rss2_head', lazylast_update);
  add_action('atom_head', lazylast_update);
  add_action('rdf_header', lazylast_update);
  if (current_user_can('list_users'))
  {
    add_action('admin_menu', lazylast_menu);
  }
}

add_action('plugins_loaded', 'lazylast_init');

?>